import random
import httpx
import json
import base64
import time
from typing_extensions import Unpack

from zzupy.log import logger
from zzupy.typing import DeviceParams
from zzupy.utils import get_sign, _kget
from zzupy.supwisdom import Supwisdom
from zzupy.ecard import eCard
from zzupy.network import Network

class ZZUPy:
    def __init__(self, usercode: str,password: str,log: bool=False):
        """
        初始化一个 ZZUPy 对象
        :param usercode: 学号
        :type usercode: str
        :param password: 密码
        :type password: str
        :param log: 是否启用日志
        :type log: bool
        """
        self._userToken = None
        self._dynamicSecret = "supwisdom_eams_app_secret"
        self._dynamicToken = None
        self._refreshToken = None
        self._name = None
        self._isLogged = False
        self._logEnabled =log
        self._DeviceParams = {}
        self._DeviceParams["deviceName"] = ""
        self._DeviceParams["deviceId"] = ""
        self._DeviceParams["deviceInfo"] = ""
        self._DeviceParams["deviceInfos"] = ""
        self._DeviceParams["userAgentPrecursor"] = ""
        self._userCode = usercode
        self._password=password
        # 初始化类
        self.Network = Network(self)
        self.eCard = eCard(self)
        self.Supwisdom = Supwisdom(self)

    def _set_params_from_password_login(self, res: str):
        try:
            self._userToken = json.loads(res)["data"]["idToken"]
            # 我也不知道 refreshToken 有什么用，但先存着吧
            self._refreshToken = json.loads(res)["data"]["refreshToken"]
        except:
            logger.error("LoginFailed")

    def _set_params_from_login_token(self, res: str):
        try:
            self._dynamicSecret = json.loads(base64.b64decode(json.loads(res)["business_data"]))["secret"]
            self._dynamicToken = json.loads(base64.b64decode(json.loads(res)["business_data"]))["token"]
            self._name = json.loads(base64.b64decode(json.loads(res)["business_data"]))["user_info"]["user_name"]
        except:
            logger.error("LoginFailed")

    def set_device_params(self, **kwargs: Unpack[DeviceParams]):
        """
        设置设备参数。这些参数都需要抓包获取，但其实可有可无，因为目前并没有观察到相关风控机制

        :param str deviceName: 设备名 ，位于 "passwordLogin" 请求的 User-Agent 中，组成为 '{appVersion}({deviceName})'
        :param str deviceId: 设备 ID ，
        :param str deviceInfo: 设备信息，位于名为 "X-Device-Info" 的请求头中
        :param str deviceInfos: 设备信息，位于名为 "X-Device-Infos" 的请求头中
        :param str userAgentPrecursor: 设备 UA 前体 ，只需要包含 "SuperApp" 或 "uni-app Html5Plus/1.0 (Immersed/38.666668)" 前面的部分
        """
        self._DeviceParams["deviceName"] = _kget(kwargs, "deviceName", "")
        self._DeviceParams["deviceId"] = _kget(kwargs, "deviceId", "")
        self._DeviceParams["deviceInfo"] = _kget(kwargs, "deviceInfo", "")
        self._DeviceParams["deviceInfos"] = _kget(kwargs, "deviceInfos", "")
        self._DeviceParams["userAgentPrecursor"] = _kget(kwargs, "deviceInfos", "")
        if self._DeviceParams["userAgentPrecursor"].endswith(" "):
            self._DeviceParams["userAgentPrecursor"] = self._DeviceParams["userAgentPrecursor"]
        else:
            self._DeviceParams["userAgentPrecursor"] = self._DeviceParams["userAgentPrecursor"] + " "
        # self.DeviceParamsSet = True

    def login(self, appVersion: str="SWSuperApp/1.0.33",appId: str="com.supwisdom.zzu", osType: str="android")-> tuple:
        """
        通过学号和密码登录

        :param str userCode: 学号
        :param str password: 密码
        :param str appVersion: APP 版本 ，一般类似 "SWSuperApp/1.0.33" ，可自行更新版本号，但详细数据需要抓包获取,位于 "passwordLogin" 请求的 User-Agent 中，也可随便填或空着，目前没有观察到相关风控机制。
        :param str appId: APP 包名，一般不需要修改
        :param str osType: 系统类型，一般不需要修改
        :return: (usercode,name) 由学号和姓名组成的元组
        :rtype: tuple
        """
        headers = {
            'User-Agent': f'{appVersion}({self._DeviceParams["deviceName"]})',
            'Connection': 'Keep-Alive',
            'Accept-Encoding': 'gzip',
        }
        response = httpx.post(
            f'https://token.s.zzu.edu.cn/password/passwordLogin?username={self._userCode}&password={self._password}&appId={appId}&geo&deviceId={self._DeviceParams["deviceId"]}&osType={osType}&clientId&mfaState',
            headers=headers,
        )
        self._set_params_from_password_login(response.text)
        cookies = {
            'userToken': self._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
            'SVRNAME': 'ws1',
        }

        headers = {
            'User-Agent': self._DeviceParams["userAgentPrecursor"] + "SuperApp",
            'Accept': 'application/json, text/plain, */*',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/x-www-form-urlencoded',
            'sec-ch-ua': '"Not/A)Brand";v="8", "Chromium";v="126", "Android WebView";v="126"',
            'sec-ch-ua-mobile': '?1',
            'sec-ch-ua-platform': '"Android"',
            'Origin': 'https://jw.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://jw.v.zzu.edu.cn/app-web/',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
            'Cookie': f'userToken={self._userToken}; Domain=.zzu.edu.cn; Path=/; SVRNAME=ws1',
        }
        data = {
            'random': int(random.uniform(10000, 99999)),
            'timestamp': int(round(time.time() * 1000)),
            'userToken': self._userToken
        }
        # 计算 sign 并将其加入 data
        params = ""
        for key in data.keys():
            params += f"{key}={data[key]}&"
        params = params[:-1]
        sign = get_sign(self._dynamicSecret, params)
        data["sign"] = sign

        response = httpx.post(
            'https://jw.v.zzu.edu.cn/app-ws/ws/app-service/super/app/login-token',
            cookies=cookies,
            headers=headers,
            data=data,
        )
        self._set_params_from_login_token(response.text)
        self._isLogged = True
        self.eCard._get_eacrd_access_token()
        return self._userCode, self._name



import base64
import datetime
import json
import random
import time
import httpx

from zzupy.utils import get_sign


class Supwisdom:
    def __init__(self, parent):
        self._parent = parent

    def getCoursesJson(self, start_date):
        """
        获取课程表

        :param str start_date: 课表的开始日期，格式必须为 %Y-%m-%d ，且必须为某一周周一，否则课表会时间错乱
        :return: 返回课程表数据，格式为json
        :rtype: str
        """
        cookies = {
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
            'SVRNAME': 'ws1',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + "SuperApp",
            'Accept': 'application/json, text/plain, */*',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/x-www-form-urlencoded',
            'sec-ch-ua': '"Not/A)Brand";v="8", "Chromium";v="126", "Android WebView";v="126"',
            'sec-ch-ua-mobile': '?1',
            'token': self._parent._dynamicToken,
            'sec-ch-ua-platform': '"Android"',
            'Origin': 'https://jw.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://jw.v.zzu.edu.cn/app-web/',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        data = {
            'biz_type_id': '1',
            'end_date': (datetime.datetime.strptime(start_date, "%Y-%m-%d") + datetime.timedelta(days=6)).strftime("%Y-%m-%d"),
            'random': int(random.uniform(10000, 99999)),
            'semester_id': '152',
            'start_date': start_date,
            'timestamp': int(round(time.time() * 1000)),
            'token': self._parent._userToken,
        }

        params = ""
        for key in data.keys():
            params += f"{key}={data[key]}&"
        params = params[:-1]
        sign = get_sign(self._parent._dynamicSecret, params)
        data["sign"] = sign

        response = httpx.post(
            'https://jw.v.zzu.edu.cn/app-ws/ws/app-service/student/course/schedule/get-course-tables',
            cookies=cookies,
            headers=headers,
            data=data,
        )
        coursesJson = (base64.b64decode(json.loads(response.text)["business_data"])).decode('utf-8')
        sorted_courses_json = sorted(json.loads(coursesJson),key=lambda x: (x['date'], datetime.datetime.strptime(x['start_time'], "%H:%M")))
        return json.dumps(sorted_courses_json).encode('utf-8').decode('unicode_escape')






import sys
import loguru
logger = loguru.logger


default_format: str = (
    "<g>{time:MM-DD HH:mm:ss}</g> "
    "[<lvl>{level}</lvl>] "
    "<c><u>{name}</u></c> | "
    # "<c>{function}:{line}</c>| "
    "{message}"
)
"""默认日志格式"""


logger_id = logger.add(
    sys.stdout,
    level=0,
    diagnose=False,
    filter="WARNING",
    format=default_format,
)




import hashlib
import socket
import psutil
import gmalg
from Crypto.Util.Padding import unpad


def get_sign(dynamicSecret, params):
    """
    获取sign值

    :param str dynamicSecret: login后自动获取，来自 login-token 请求
    :param str params: URL请求参数
    :return: sign值
    :return_type: str
    """
    paramsDict = {}
    for param in params.split("&"):
        if param.split("=")[0] == "timestamp":
            timestamp = param.split("=")[1]
        elif param.split("=")[0] == "random":
            random = param.split("=")[1]
        else:
            paramsDict[param.split("=")[0]] = param.split("=")[1]
    paramsDict = dict(sorted(paramsDict.items()))
    original = f"{dynamicSecret}|"
    for key in paramsDict:
        original += f"{paramsDict[key]}|"
    original += f"{timestamp}|{random}"
    sign = hashlib.md5(original.encode("utf-8")).hexdigest().upper()
    return sign


def _kget(kwargs, key, default=None):
    return kwargs[key] if key in kwargs else default


def get_ip_by_interface(interface):
    addresses = psutil.net_if_addrs()
    if interface in addresses:
        for addr in addresses[interface]:
            if addr.family == socket.AF_INET:
                return addr.address
    return None


def sm4_decrypt_ecb(ciphertext: bytes, key: bytes):
    """
    SM4 解密，ECB模式

    :param bytes ciphertext: 密文
    :param bytes key: 密钥
    :return: 明文 Hex
    :return_type: str
    """
    sm4 = gmalg.SM4(key)
    block_size = 16
    decrypted_padded = b""
    for i in range(0, len(ciphertext), block_size):
        block = ciphertext[i:i + block_size]
        decrypted_padded += sm4.decrypt(block)
    decrypted = unpad(decrypted_padded, block_size)
    return decrypted.decode()



import base64
import random
import httpx
from fake_useragent import UserAgent

from zzupy.utils import get_ip_by_interface

class Network:
    def __init__(self,parent):
        self._parent=parent

    def portal_auth(self,interface: str="",baseurl='http://10.2.7.8:801',ua=UserAgent()):
        """
        进行校园网认证

        :param str interface: 网络接口名
        :param str baseurl: PortalAuth Server URL。一般无需修改
        :param str ua: User-Agent
        """
        if interface=="":
            local_client = httpx.Client()
        else:
            transport = httpx.HTTPTransport(local_address=get_ip_by_interface(interface))
            local_client = httpx.Client(transport=transport)
        self._chkstatus(local_client,baseurl,ua)
        self._loadConfig(local_client,interface,baseurl,ua)
        self._auth(local_client,interface,baseurl,ua)

    def _auth(self,client,interface,baseURL, ua,):
        headers = {
            'Accept': '*/*',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
            'Connection': 'keep-alive',
            'Referer': 'http://10.2.7.8/',
            'User-Agent': ua,
        }
        params = [
            ('callback', 'dr1003'),
            ('login_method', '1'),
            ('user_account', f',0,{self._parent.userCode}'),
            ('user_password', base64.b64encode(self._parent._password.encode()).decode()),
            ('wlan_user_ip', get_ip_by_interface(interface)),
            ('wlan_user_ipv6', ''),
            ('wlan_user_mac', '000000000000'),
            ('wlan_ac_ip', ''),
            ('wlan_ac_name', ''),
            ('jsVersion', '4.2.1'),
            ('terminal_type', '1'),
            ('lang', 'zh-cn'),
            ('v', str(random.randint(500, 10499))),
            ('lang', 'zh'),
        ]
        response = client.get(f"{baseURL}/eportal/portal/login", params=params, headers=headers)
        return interface, response.text

    def _chkstatus(self,client, baseURL, ua):
        headers = {
            'Accept': '*/*',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
            'Connection': 'keep-alive',
            'Referer': 'http://10.2.7.8/a79.htm',
            'User-Agent': ua,
        }

        params = {
            'callback': 'dr1002',
            'jsVersion': '4.X',
            'v': str(random.randint(500, 10499)),
            'lang': 'zh',
        }
        client.get(f"{baseURL}/drcom/chkstatus", params=params, headers=headers)

    def _loadConfig(self,client,interface, baseURL, ua):
        headers = {
            'Accept': '*/*',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
            'Connection': 'keep-alive',
            'Referer': 'http://10.2.7.8/',
            'User-Agent': ua,
        }

        params = {
            'callback': 'dr1001',
            'program_index': '',
            'wlan_vlan_id': '1',
            'wlan_user_ip': base64.b64encode(get_ip_by_interface(interface).encode()).decode(),
            'wlan_user_ipv6': '',
            'wlan_user_ssid': '',
            'wlan_user_areaid': '',
            'wlan_ac_ip': '',
            'wlan_ap_mac': '000000000000',
            'gw_id': '000000000000',
            'jsVersion': '4.X',
            'v': str(random.randint(500, 10499)),
            'lang': 'zh',
        }
        client.get(f"{baseURL}/eportal/portal/page/loadConfig", params=params, headers=headers)






import base64
import json
import time
import httpx
import gmalg
from zzupy.utils import sm4_decrypt_ecb

class eCard:
    def __init__(self, parent):
        self._parent = parent
        self._eCardAccessToken = ""
        self._JSessionID = ""
        self._tid=""

    def _get_eacrd_access_token(self):
        cookies = {
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + 'SuperApp',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'sec-ch-ua-platform': '"Android"',
            'Upgrade-Insecure-Requests': '1',
            'x-id-token': self._parent._userToken,
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'none',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-User': '?1',
            'Sec-Fetch-Dest': 'document',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        params = {
            'host': '11',
            'org': '2',
            'token': self._parent._userToken,
        }

        response = httpx.get('https://ecard.v.zzu.edu.cn/server/auth/host/open', params=params, cookies=cookies,
                             headers=headers, follow_redirects=False)
        self._JSessionID = response.headers['set-cookie'].split("=")[1].split(";")[0]
        self._tid = response.headers['location'].split('=')[1].split("&")[0]
        cookies = {
            'JSESSIONID': self._JSessionID,
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + 'SuperApp',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'sec-ch-ua-platform': '"Android"',
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={self._tid}&orgId=2',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        data = {
            'tid': self._tid,
        }

        response = httpx.post('https://ecard.v.zzu.edu.cn/server/auth/getToken', cookies=cookies, headers=headers,
                              json=data)
        self._eCardAccessToken = json.loads(response.text)["resultData"]["accessToken"]

    def recharge_electricity(self, room: str,paypasswd: str) -> tuple:
        """
        为 room 充值电费

        :param str room: 宿舍房间。理论上空调和照明均支持.格式应为 “areaid-buildingid--unitid-roomid”，可通过get_area_dict(),get_building_dict(),get_unit_dict(),get_room_dict()获取
        :param str paypasswd: 支付密码
        :return: (bool,msg) bool 为 True即为成功，msg 为服务端响应信息
        :rtype: tuple
        """
        cookies = {
            'JSESSIONID': self._JSessionID,
        }

        headers = {
            'Accept': '*/*',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
            'Authorization': self._eCardAccessToken,
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Content-Type': 'application/json',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'Pragma': 'no-cache',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={self._tid}&orgId=2',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-origin',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36',
            'sec-ch-ua': '"Google Chrome";v="131", "Chromium";v="131", "Not_A Brand";v="24"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
        }
        response = httpx.post('https://ecard.v.zzu.edu.cn/server/auth/getEncrypt', cookies=cookies, headers=headers)
        pay_id=json.loads(response.text)["resultData"]["id"]
        encrypted_public_key=json.loads(response.text)["resultData"]["publicKey"]
        # 解密被加密的公钥
        public_key=sm4_decrypt_ecb(base64.b64decode(encrypted_public_key), bytes.fromhex("773638372d392b33435f48266a655f35"))
        # 请求体明文
        json_data = {
            "utilityType": "electric",
            "payCode": "06",
            "password": paypasswd,
            "amt": "10",
            "timestamp": int(round(time.time() * 1000)),
            "bigArea": "",
            "area": room.split("--")[0].split("-")[0],
            "building": room.split("--")[0].split("-")[1],
            "unit": "",
            "level": room.split("--")[1].split("-")[0],
            "room": room,
            "subArea": "",
            "customfield": {
            }
        }
        json_string = json.dumps(json_data, separators=(',', ':'))
        # 加密 params
        sm2=gmalg.SM2(pk=bytes.fromhex(public_key))
        encrypted_params = sm2.encrypt(json_string.encode())
        data = {
            "id": pay_id,
            "params": (encrypted_params.hex())[2:]
        }
        response = httpx.post('https://ecard.v.zzu.edu.cn/server/utilities/pay', cookies=cookies, headers=headers, json=data)
        return json.loads(response.text)["success"],json.loads(response.text)["message"]
    
    
    def get_balance(self) -> float:
        """
        获取校园卡余额

        :return: 校园卡余额
        :rtype: float
        """

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + "uni-app Html5Plus/1.0 (Immersed/38.666668)",
            'Connection': 'Keep-Alive',
            'Accept': 'application/json',
            'Accept-Encoding': 'gzip',
            'X-Device-Info': self._parent._DeviceParams["deviceInfo"],
            'X-Device-Infos': self._parent._DeviceParams["deviceInfos"],
            'X-Id-Token': self._parent._userToken,
            'X-Terminal-Info': 'app',
            'Content-Type': 'application/x-www-form-urlencoded',
        }

        response = httpx.get('https://info.s.zzu.edu.cn/portal-api/v1/thrid-adapter/get-person-info-card-list',headers=headers)
        return float(json.loads(response.text)["data"][1]["amount"])

    def get_area_dict(self) -> dict:
        """
        获取区域的字典

        :return: 区域字典
        :rtype: dict
        """
        cookies = {
            'JSESSIONID': self._JSessionID,
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + 'SuperApp',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'sec-ch-ua-platform': '"Android"',
            'Authorization': self._eCardAccessToken,
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={self._tid}&orgId=2',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        data = {
            'utilityType': 'electric',
            'locationType': 'bigArea',
            'bigArea': '',
            'area': '',
            'building': '',
            'unit': '',
            'level': '',
            'room': '',
            'subArea': '',
        }

        response = httpx.post('https://ecard.v.zzu.edu.cn/server/utilities/location', cookies=cookies,
                              headers=headers, json=data)
        AreaDict = {}
        for i in range(len(json.loads(response.text)["resultData"]["locationList"])):
            AreaDict[json.loads(response.text)["resultData"]["locationList"][i]["id"]] = \
                json.loads(response.text)["resultData"]["locationList"][i]["name"]
        return AreaDict

    def get_building_dict(self, areaid: str) -> dict:
        """
        获取建筑的字典

        :param str areaid: 通过get_area_dict()获取
        :return: 建筑字典
        :rtype: dict
        """
        cookies = {
            'JSESSIONID': self._JSessionID,
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + 'SuperApp',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'sec-ch-ua-platform': '"Android"',
            'Authorization': self._eCardAccessToken,
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={self._tid}&orgId=2',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        data = {
            'utilityType': 'electric',
            'locationType': 'building',
            'bigArea': '',
            'area': areaid,
            'building': '',
            'unit': '',
            'level': '',
            'room': '',
            'subArea': '',
        }

        response = httpx.post('https://ecard.v.zzu.edu.cn/server/utilities/location', cookies=cookies,
                              headers=headers, json=data)
        BuildingDict = {}
        for i in range(len(json.loads(response.text)["resultData"]["locationList"])):
            BuildingDict[json.loads(response.text)["resultData"]["locationList"][i]["id"]] = \
                json.loads(response.text)["resultData"]["locationList"][i]["name"]
        return BuildingDict

    def get_unit_dict(self, areaid: str, buildingid: str) -> dict:
        """
        获取照明/空调的字典

        :param str areaid: 通过get_unit_dict()获取
        :param str buildingid: 通过get_building_dict()获取
        :return: 照明/空调字典
        :rtype: dict
        """
        cookies = {
            'JSESSIONID': self._JSessionID,
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + 'SuperApp',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'sec-ch-ua-platform': '"Android"',
            'Authorization': self._eCardAccessToken,
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={self._tid}&orgId=2',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        data = {
            'utilityType': 'electric',
            'locationType': 'unit',
            'bigArea': '',
            'area': areaid,
            'building': buildingid,
            'unit': '',
            'level': '',
            'room': '',
            'subArea': '',
        }

        response = httpx.post('https://ecard.v.zzu.edu.cn/server/utilities/location', cookies=cookies,
                              headers=headers, json=data)
        UnitDict = {}
        for i in range(len(json.loads(response.text)["resultData"]["locationList"])):
            UnitDict[json.loads(response.text)["resultData"]["locationList"][i]["id"]] = \
                json.loads(response.text)["resultData"]["locationList"][i]["name"]
        return UnitDict

    def get_room_dict(self, areaid: str, buildingid: str, unitid: str) -> dict:
        """
        获取房间的字典

        :param str areaid: 通过get_area_dict()获取
        :param str buildingid: 通过get_building_dict()获取
        :param str unitid: 通过get_unit_dict()获取
        :return: 房间字典
        :rtype: dict
        """
        cookies = {
            'JSESSIONID': self._JSessionID,
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + 'SuperApp',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'sec-ch-ua-platform': '"Android"',
            'Authorization': self._eCardAccessToken,
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={self._tid}&orgId=2',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        data = {
            'utilityType': 'electric',
            'locationType': 'room',
            'bigArea': '',
            'area': areaid,
            'building': buildingid,
            'unit': '',
            'level': unitid,
            'room': '',
            'subArea': '',
        }

        response = httpx.post('https://ecard.v.zzu.edu.cn/server/utilities/location', cookies=cookies,
                              headers=headers, json=data)
        RoomDict = {}
        for i in range(len(json.loads(response.text)["resultData"]["locationList"])):
            RoomDict[json.loads(response.text)["resultData"]["locationList"][i]["id"]] = \
                json.loads(response.text)["resultData"]["locationList"][i]["name"]
        return RoomDict

    def get_remaining_power(self, room: str) -> float:
        """
        获取剩余电量

        :param str room: 格式应为 “areaid-buildingid--unitid-roomid”，可通过get_area_dict(),get_building_dict(),get_unit_dict(),get_room_dict()获取
        :return: 剩余电量
        :rtype: float
        """
        # bind 请求，似乎不必要还会花费大量时间
        '''
        AreaDict=self.get_area_dict()
        BuildingDict=self.get_building_dict(room.split("--")[0].split("-")[0])
        UnitDict=self.get_unit_dict(room.split("--")[0].split("-")[0],room.split("--")[0].split("-")[1])
        RoomDict=self.get_room_dict(room.split("--")[0].split("-")[0],room.split("--")[0].split("-")[1],room.split("--")[1].split("-")[0])

        cookies = {
            'JSESSIONID': self._JSessionID,
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }


        headers = {
            'User-Agent':  self._parent._DeviceParams["userAgentPrecursor"]+'SuperApp',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'sec-ch-ua-platform': '"Android"',
            'Authorization': self._eCardAccessToken,
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={_tid}&orgId=2',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        json_data = {
            'bigArea': '',
            'bigAreaName': '',
            'area': room.split("--")[0].split("-")[0],
            'areaName': AreaDict[room.split("--")[0].split("-")[0]],
            'building': room.split("--")[0].split("-")[1],
            'buildingName': BuildingDict[room.split("--")[0].split("-")[1]],
            'unit': '',
            'unitName': None,
            'level': room.split("--")[1].split("-")[0],
            'levelName': UnitDict[room.split("--")[1].split("-")[0]],
            'room': room,
            'roomName': RoomDict[room],
            'subArea': '',
            'subAreaName': None,
            'utilityType': 'electric',
            'locationType': 'room',
        }

        response = httpx.post('https://ecard.v.zzu.edu.cn/server/utilities/bind', cookies=cookies, headers=headers,json=json_data)
        '''

        cookies = {
            'JSESSIONID': self._JSessionID,
            'userToken': self._parent._userToken,
            'Domain': '.zzu.edu.cn',
            'Path': '/',
        }

        headers = {
            'User-Agent': self._parent._DeviceParams["userAgentPrecursor"] + 'SuperApp',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'sec-ch-ua-platform': '"Android"',
            'Authorization': self._eCardAccessToken,
            'sec-ch-ua': '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            'sec-ch-ua-mobile': '?1',
            'Origin': 'https://ecard.v.zzu.edu.cn',
            'X-Requested-With': 'com.supwisdom.zzu',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': f'https://ecard.v.zzu.edu.cn/?tid={self._tid}&orgId=2',
            'Accept-Language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        }

        data = {
            'utilityType': 'electric',
            'bigArea': '',
            'area': room.split("--")[0].split("-")[0],
            'building': room.split("--")[0].split("-")[1],
            'unit': '',
            'level': room.split("--")[1].split("-")[0],
            'room': room,
            'subArea': '',
        }

        response = httpx.post('https://ecard.v.zzu.edu.cn/server/utilities/account', cookies=cookies,headers=headers, json=data)
        return json.loads(response.text)["resultData"]["templateList"][3]["value"]

